
#ifndef TEST_LAYER_H
#define TEST_LAYER_H


#include <QString>
#include <QtTest>
#include "AutoTest.h"

class Object;


class TestLayer : public QObject
{
    Q_OBJECT

public:
    TestLayer();

private slots:
    void initTestCase();
    void cleanupTestCase();

    void testCase1();
    void testLayerType();
    
    void testAddNewKeyAtBitmap();
    void testAddNewKeyAtVector();

    void testRemoveKeyFrame();
    void testGetMaxFramePosition();
    void testGetFirstFramePosition();
    void testHasKeyFrameAtPosition();

    void testPreviousKeyFramePosition();
    void testNextKeyFramePosition();


private:
    Object* m_pObject = nullptr;
};

DECLARE_TEST( TestLayer )

#endif // TEST_LAYER_H
